import React from "react";
import { useAlert } from "./useAlert";
import { useHistory } from "react-router";
import { httpStatusCodes } from "../constants";
// import { auth } from "../services";
import { useApp } from "./useApp";
import { useHttpProps, innerFetch, useHttpResponse } from "../types";

const { REACT_APP_API_URL: baseUrl } = process.env;

const defaultHeaders = {
  "Content-Type": "application/json",
  Accept: "application/json",
};

const makeHeaders = (token: string | null) =>
  token
    ? {
        ...defaultHeaders,
        Authorization: `Bearer ${token}`,
      }
    : defaultHeaders;

const paramsToQuery = (params: any) =>
  Object.keys(params)
    .map(
      (key) => encodeURIComponent(key) + "=" + encodeURIComponent(params[key])
    )
    .join("&");

const capitalize = (s: string) => s.charAt(0).toUpperCase() + s.slice(1);

export function useHttp({
  req = "GET",
  url = "",
  params = null,
  body = null,
  triggerAlert = false,
}: useHttpProps): [
  useHttpResponse | null,
  boolean,
  string | null,
  (showAlert: any, inlineParams?: any) => Promise<void>
] {
  const { showAlert } = useAlert();
  const { token } = useApp();
  const [response, setResponse] = React.useState<useHttpResponse | null>(null);
  const [error, setError] = React.useState<string | null>(null);
  const [loading, setLoading] = React.useState(true);
  const history = useHistory();

  const refresh = React.useCallback(
    async (showAlert, inlineParams = {}) => {
      try {
        if (!url || !params || !req) {
          setResponse(null);
          setError(null);
          setLoading(true);
          return;
        }
        if (inlineParams.loading === false) {
          setLoading(() => false);
        } else {
          setLoading(() => true);
        }
        const fetchReq: innerFetch = {
          method: req,
          headers: makeHeaders(token),
          body: null,
        };
        if (body) {
          const serializedBody = JSON.stringify(body);
          fetchReq.body = serializedBody;
        }
        const paramsFinal = { ...params, ...inlineParams };
        const str = `${baseUrl}${url}${
          params && Object.keys(paramsFinal).length > 0
            ? `?${paramsToQuery(paramsFinal)}`
            : ""
        }`;
        const httpRes = await fetch(str, fetchReq);
        const resBody = await httpRes.json();
        switch (httpRes.status) {
          case httpStatusCodes.default.OK:
            setResponse(resBody);
            setError(null);
            triggerAlert &&
              showAlert({
                severity: "success",
                message: resBody.mensaje
                  ? capitalize(resBody.mensaje)
                  : "Solicitud completada correctamente!",
              });
            break;
          case httpStatusCodes.default.BAD_REQUEST:
            window["scrollTo"]({ top: 0, behavior: "smooth" });
            setError(resBody.errores);
            triggerAlert &&
              showAlert({
                severity: "warning",
                message: resBody.mensaje
                  ? capitalize(resBody.mensaje)
                  : "Datos erróneos o inválidos.",
              });
            break;
          case httpStatusCodes.default.UNAUTHORIZED:
            history.push("/no-autorizado");
            break;
          case httpStatusCodes.default.INTERNAL_SERVER_ERROR:
          default:
            triggerAlert &&
              showAlert({
                severity: "error",
                message: resBody.mensaje
                  ? capitalize(resBody.mensaje)
                  : "Ocurrió un error en el servidor.",
              });
        }
      } catch (error) {
        triggerAlert &&
          showAlert({
            severity: "error",
            message: "No se pudo establecer conexión con el servidor.",
          });
        console.error(error);
      } finally {
        setLoading(false);
      }
    },
    [body, params, req, url, triggerAlert, history, token]
  );

  React.useEffect(() => {
    let mounted = true;
    if (mounted) {
      refresh(showAlert);
    }
    return () => {
      mounted = false;
    };
  }, [refresh, showAlert]);

  return React.useMemo(
    () => [response, loading, error, refresh],
    [response, loading, error, refresh]
  );
}
